package org.amse.asves.skinCreator.test;

import java.awt.Color;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;
import java.util.zip.*;


import junit.framework.*;
import org.amse.asves.skinCreator.*;
import org.amse.asves.skinCreator.exceptions.*;
import org.amse.asves.skinCreator.model.*;
import org.amse.asves.skinCreator.writer.*;
import org.amse.asves.skinCreator.wsz.*;
import org.amse.asves.skinCreator.model.impl.*;

public class TestModel extends TestCase{
	
	private String readStringFromFile(int testNumber) {
    	Reader reader = null;
		String fileName = "";
    	try{
      		fileName = "org/amse/asves/skinCreator/test/tests/test" + testNumber + ".txt";
      		reader = new BufferedReader(new FileReader(fileName));
    	}catch(FileNotFoundException e){
      		fail(fileName + "not found");
    	}
    	StringBuffer sb = new StringBuffer();
    	try{
      		try{
        		char[] buffer = new char[1024];
        		int counter;
        		while((counter = reader.read(buffer)) != -1){
          			sb.append(new String(buffer, 0, counter));
        		}
      		}finally{
        		reader.close();
      		}
    	}catch (IOException e){
    	}
		return sb.toString();
	}
  	
	
	
	
	
	
	private RGBColor tryColor(int a, int b, int c){
		RGBColor temp;
		try{
			temp = new RGBColor(a,b,c);
			return temp;
		}catch(InvalidValueException e){
			fail();
			return null;
		}
	}
	
	
	static IProject myProject;
	static ISkin mySkin;
	static IWidget myTitleBarWidget;
	static File myDefaultSkin;
	static SkinFilesManager myExtractor;
	
	
	public void test1(){
		try{
			InputStream input = TestModel.class.getResourceAsStream("testSkin.wsz");
			String inputFileWithExtention = readStringFromFile(1);
			myDefaultSkin = File.createTempFile(inputFileWithExtention, ".skc");
			myDefaultSkin.deleteOnExit();
			
			Main.createTempFileFromStream(input, myDefaultSkin);
			
			myExtractor = new SkinFilesManager(myDefaultSkin);
			assertEquals(SkinFilesManager.getTempSourceFileName(myDefaultSkin.getName()),
					inputFileWithExtention);
		}catch(Exception e){
			fail("exception found - " + e.getMessage());
		}
	}
	
	public void test2(){
		String entryName = "";
		try{
			ZipFile zip = myExtractor.getDefaultSkin();
			List<ZipEntry> entries = myExtractor.getEntries(zip);
		
			for (ZipEntry entry : entries)
				if (entry.getName().toUpperCase().equals("PLEDIT.TXT")){
					entryName = entry.getName();
				}
		}catch(Exception e){
			fail("exception found - " + e.getMessage());
		}
		assertEquals(entryName,readStringFromFile(2));
	}
	
	public void test3(){
		try{
			myProject = Main.initializeProject(myExtractor);
			mySkin = myProject.getSkin();
			myTitleBarWidget = mySkin.getWidget(WidgetName.MAINTITLEBAR);
		}catch(Exception e){
			fail("exception found - " + e.getMessage());
		}
	}
	/**
	 *    ContainerName
	 *
	 */
	public void test4(){
		assertEquals(ContainerName.NUMBERS, ContainerName.NUMS_EX.getEmergencyContainer());
	}

	public void test5(){
		assertEquals(ContainerName.SHUFREP.getFileNameU(), "SHUFREP.BMP");
	}
	
	/**
	 *   IProject
	 *
	 */
	public void test6(){
		IProject project = new Project(myProject.getSkin());
		assertEquals(myProject.getSkin(),project.getSkin());
	}
	
	public void test7(){
		ISkin skin = null;
		try{
			skin = new Skin(null);
		}catch(CodeError e){
			fail();
		}
		IProject project = new Project(skin);
		IProject project2 = new Project(skin);
		assertEquals(project2.getSkin(),project.getSkin());
	}
	
	/**
	 *   WidgetName
	 *
	 */
	public void test8(){
		List<WidgetName> list = WidgetName.getGenText();
		assertTrue(list.contains(WidgetName.GEN_TEXT_A));
	}
	
	public void test9(){
		List<WidgetName> list = WidgetName.getGenTextActive();
		assertTrue(list.contains(WidgetName.GEN_TEXT_A_act));
	}
	
	/**
	 *   ISkin
	 *
	 */
	public void test10(){
		RGBColor color = tryColor(10,11,11);
		mySkin.setViscolorColor(2,color);
		
		assertEquals(readStringFromFile(10),mySkin.getViscolorColor(2).toString());
	}
	
	public void test11(){
		RGBColor color = tryColor(255,11,11);
		mySkin.setPleditColor(PleditParameterName.NormalBG ,color);
		
		assertEquals(readStringFromFile(11),
				mySkin.getPleditColor(PleditParameterName.NormalBG).toString());
	}
	
	public void test12(){
		RGBColor color = tryColor(13,60,156);
		mySkin.setScrollBGColor(color);
		
		assertEquals(readStringFromFile(12),
				mySkin.getScrollBGColor().toString());
	}
	
	public void test13(){
		mySkin.setFont("MS SANS SERIF");
		
		assertEquals(readStringFromFile(13),
				mySkin.getFont().toString());
	}
	
	/**
	 *   IWidget
	 */
	public void test14(){
		myTitleBarWidget.setPicture(new BufferedImage(120,240,12));
		
		assertEquals(readStringFromFile(14), "" + 
				myTitleBarWidget.getWidth() + ", " + 
				myTitleBarWidget.getHeight());
	}
	
	public void test15(){
		myTitleBarWidget.setPicture(new BufferedImage(120,240,12));
		assertEquals(myTitleBarWidget.getWidth(),myTitleBarWidget.getPicture().getWidth());
	}
	
	public void test16(){
		assertEquals(myTitleBarWidget.getName(),WidgetName.MAINTITLEBAR);
	}
	
	public void test17(){
		assertEquals(myTitleBarWidget.toString(),readStringFromFile(17));
	}
	
	/**
	 *   PleditParameterName
	 */
	public void test18(){
		assertEquals(PleditParameterName.getValidParameterName("SeLEctEDbG").toString(),
				readStringFromFile(18));
	}
	
	public void test19(){
		assertEquals(PleditParameterName.getValidParameterName("SeLEctE DbG"),
				null);
	}
	
	public void test20(){
		assertTrue(PleditParameterName.getAllParameterNames().contains(
				PleditParameterName.NormalBG));
	}
	
	public void test21(){
		assertTrue(PleditParameterName.getAllColorParameterNames().contains(
				PleditParameterName.NormalBG));
	}
	
	public void test22(){
		assertFalse(PleditParameterName.getAllColorParameterNames().contains(
				PleditParameterName.Font));
	}
	
	/**
	 *   Rectangle
	 */
	public void test23(){
		assertEquals((new Rectangle()).area(), 0);
	}
	
	public void test24(){
		assertEquals((new Rectangle(16, 14, 436, 664)).area(), 273000);
	}
	
	public void test25(){
		assertTrue((new Rectangle(16, 14, 436, 664)).contains(16, 436));
	}
	
	public void test26(){
		assertTrue((new Rectangle(16, 14, 436, 664)).contains(17, 436));
	}
	
	public void test27(){
		assertEquals((new Rectangle(436, 14, 16, 664)).area(),
				(new Rectangle(16, 14, 436, 664)).area());
	}
	
	public void test28(){
		assertEquals((new Rectangle(16, 325, 436, 664)).AX(),
				(new Rectangle(16, 14, 664, 434)).AX());
	}
	
	public void test29(){
		assertFalse((new Rectangle(15, 17, 436, 664)).BX() ==
				(new Rectangle(16, 14, 664, 436)).BX());
	}
	
	public void test30(){
		assertEquals((new Rectangle(15, 17, 436, 664)).toString(),
				readStringFromFile(30));
	}
	
	public void test31(){
		assertEquals((new Rectangle(15, 17, 435, 664)).getCenterX(), 225);
	}
	
	public void test32(){
		int centerX = (new Rectangle(15, 17, 435, 664)).getCenterX();
		int centerY = (new Rectangle(15, 17, 435, 664)).getCenterY();
		assertTrue((new Rectangle(15, 17, 435, 664)).contains(centerX, centerY));
	}
	
	/**
	 *   RGBColor
	 */
	public void test33(){
		RGBColor color = tryColor(13, 12, 11);
		assertEquals(new Color(13,12,11), color.getColor());
	}
	
	public void test34(){
		RGBColor color = tryColor(13, 12, 13);
		assertEquals(readStringFromFile(34), color.toString());
	}
	
	public void test35(){
		RGBColor color = new RGBColor();
		try{
			color.setRGBColor(-45, 555, 32);
			fail();
		}catch(InvalidValueException e){
			assertTrue(true);
		}
	}
	
	public void test36(){
		RGBColor color = new RGBColor(); 
		try{
			color.encodeFromDecimal("45, 35, 145");
		}catch(InvalidValueException e){
			fail();
		}
		assertEquals(color.getColor(), new Color(45, 35, 145));
	}
	
	public void test37(){
		RGBColor color = new RGBColor(); 
		assertEquals(color.toString(), color.getRedMask() + ", " + 
				color.getBlueMask() + ", " + color.getGreenMask());
	}
	
	public void test38(){
		RGBColor color = new RGBColor();
		try{
		    color.encodeFromHexadecimal("#AAFF00");
		}catch(InvalidValueException e){
			fail();
		}
		assertEquals(color.toString(), readStringFromFile(38));
	}
	
	public void test39(){
		RGBColor color = new RGBColor();
		try{
			color.setRedMask(14);
			color.setGreenMask(255);
			color.setBlueMask(27);
		}catch(InvalidValueException e){
			fail();
		}
		assertEquals(color.codeToHexadecimal(), readStringFromFile(39));
	}
	
	/**
	 *   TxtFileName
	 *
	 */
	public void test40(){
		assertEquals(TxtFileName.REGION.getFileName(), readStringFromFile(40));
	}
	
	/**
	 *   WidgetName
	 *
	 */
	public void test41(){
		WidgetName name1 = WidgetName.REPEAT_OFF_PRESSED;
		WidgetName name2 = WidgetName.REPEAT_ON;
		assertEquals(name1.getNextStateOfButton(), name2);
	}
	
	public void test42(){
		WidgetName name2 = WidgetName.SHUFFLE_ON;
		WidgetName name3 = WidgetName.SHUFFLE_ON_PRESSED;
		assertEquals(name2.getNextStateOfButton(), name3);
	}
	
	public void test43(){
		WidgetName name1 = WidgetName.REPEAT_OFF_PRESSED;
		WidgetName name4 = WidgetName.REPEAT_OFF;
		assertEquals(name4.getNextStateOfButton(), name1);
	}
	
	public void test44(){
		WidgetName name = WidgetName.REPEAT_OFF_PRESSED;
		assertFalse(name.isGenLetter());
	}
	
	public void test45(){
		WidgetName name = WidgetName.GEN_TEXT_O;
		assertFalse(name.isTextLetter());
	}
	
	public void test46(){
		WidgetName name = WidgetName.GEN_TEXT_O;
		assertTrue(name.isGenLetter());
	}
	
	public void test47(){
		WidgetName name = WidgetName.GEN_TEXT_Z;
		assertTrue(name.isGenLetter());
	}
	
	public void test48(){
		WidgetName name = WidgetName.AVS_VIS;
		assertTrue(name.isProgram());
	}
	
	public void test49(){
		WidgetName name = WidgetName.ARROW;
		assertTrue(name.isProgram());
	}
	
	public void test50(){
		WidgetName name = WidgetName.GEN_TEXT_Z;
		assertTrue(name.getNextStateOfButton().isGenLetter());
	}
	
	/**
	 *        (Reader)
	 *       Main.initialize......
	 */
	public void test51(){
		//new WSZRead(myZip, myZip);
		assertEquals(readStringFromFile(51), "" + 
				mySkin.getWidget(WidgetName.MAIN_BG).getWidth());
	}
	
	public void test52(){
		assertEquals(readStringFromFile(52), "" + 
				mySkin.getPleditColor(PleditParameterName.Normal));
	}
	
	public void test53(){
		assertEquals(readStringFromFile(53), "" + 
				mySkin.getViscolorColor(0));
	}
	
	/**
	 *   TxtFileReader + Writer
	 */
	/**
	 *      ,  
	 *       
	 */
	public void test54(){
		try{
			SkinFilesManager manager = null;
			RGBColor setColor = new RGBColor();
		
			setColor = new RGBColor(120, 90, 87);
			mySkin.setPleditColor(PleditParameterName.MbBG, setColor);
			SkinWriter writer = new SkinWriter(mySkin, myExtractor);
		
			writer.writeAll(myDefaultSkin.getPath());
			manager = new SkinFilesManager(myDefaultSkin);
			
			//    !
			mySkin.setPleditColor(PleditParameterName.MbBG, new RGBColor(114, 0, 0));
			myProject = Main.initializeProject(manager);
			mySkin = myProject.getSkin();
			assertEquals(readStringFromFile(54), "" + mySkin.getPleditColor(PleditParameterName.MbBG));
		}catch(Exception e){
			e.printStackTrace();
			fail("exception found in read - " + e.getMessage());
		}
	}
	
	/**
	 *    
	 */
	public void test55(){
		try{
			RGBColor setColor = new RGBColor(148, 200, 17);
			
			InputStream input = TestModel.class.getResourceAsStream("testSkin.wsz");
			File currentSkin = File.createTempFile("currentSkin.wsz",".skc");
			currentSkin.deleteOnExit();
			
			Main.createTempFileFromStream(input, currentSkin);
			
			myExtractor = new SkinFilesManager(myDefaultSkin);
			myExtractor.setCurrentSkin(currentSkin);
			myProject = Main.initializeProject(myExtractor);
			mySkin = myProject.getSkin();
			mySkin.setPleditColor(PleditParameterName.NormalBG, setColor);
			
			SkinWriter writer = new SkinWriter(mySkin, myExtractor);
		
			writer.writeAll(currentSkin.getPath());
			
			//    !
			mySkin.setPleditColor(PleditParameterName.NormalBG, new RGBColor(114, 0, 0));
			
			myExtractor.refreshCurrentSkin();
			
			myProject = Main.initializeProject(myExtractor);
			mySkin = myProject.getSkin();
			
			assertEquals(setColor.toString(), "" + mySkin.getPleditColor(PleditParameterName.NormalBG));
		}catch(Exception e){
			e.printStackTrace();
			fail("exception found in read - " + e.getMessage());
		}
	}
	
	/**
	 * destructor
	 */
	public void test100(){
		myDefaultSkin.delete();
	}
}
