package org.amse.asves.skinCreator.writer;

import java.io.*;
import java.util.zip.*;
import java.util.*;
import org.amse.asves.skinCreator.exceptions.*;
import org.amse.asves.skinCreator.model.TxtFileName;
import org.amse.asves.skinCreator.model.ISkin;
import org.amse.asves.skinCreator.wsz.*;

/**
 *    zip/wsz
 *       
 * @author 
 *
 */
public final class SkinWriter {
	
	private static ISkin mySkin;
	private String myZipFileName;
	private SkinFilesManager myExtractor;
	
	/**
	 *     mySkin    @param manager
	 * @throws IOException
	 */
	private static void writeFile(TxtFileContentManager manager, File file) throws IOException{

		PrintWriter writer = new PrintWriter(file);
		manager.setContent(mySkin, writer);

	}
	/**
	 *    @param filename
	 * @throws IOException     
	 */
	private static void writeTxtFile(TxtFileName filename, File file) throws IOException{
		switch (filename){
		case VISCOLOR :{
			writeFile(new ViscolorWriter(), file);
			break;
		}
		case PLEDIT : {
			writeFile(new PleditWriter(), file);
			break;
		}
		default : {
			break;
		}
		}
	}
	
	/**
	 *     -   @param szPath
	 * @return 
	 * @throws IOException
	 */
	private static ZipOutputStream createZipOutputStream(String szPath) throws IOException{
		File tempfile;  
		ZipOutputStream zos = null;
    
		tempfile = new File(szPath);
      
		zos = new ZipOutputStream(new FileOutputStream(tempfile));
	    zos.setLevel(Deflater.DEFAULT_COMPRESSION);
		
		return zos;
	}
	
	/**
	 *      @param path
	 *    @param name   
	 *  @param zos       
	 *   @param path    
	 * @throws IOException
	 */
	private static void addFileToZip(ZipOutputStream zos, File file) throws IOException{
	    
	    ZipEntry ze;
	    String name = SkinFilesManager.getTempSourceFileName(file.getName());
	    
	    ze = new ZipEntry(name);
	    if (name.toUpperCase().equals("PLEDIT.TXT")){
	    	writeTxtFile(TxtFileName.PLEDIT, file);
	    }else{if (name.toUpperCase().equals("VISCOLOR.TXT")){
	    	writeTxtFile(TxtFileName.VISCOLOR, file);
	    	//System.out.println(file.getPath());
	    	//System.out.println(name);
	    }}
	    
	    zos.putNextEntry(ze);
	      
	    FileInputStream fis = 
	      new FileInputStream(file);
	    
	    //System.out.println(path+name);
	    
	    SkinFilesManager.copyStream(fis, zos);
	      
	    fis.close();
	    zos.closeEntry();
	}
	
	/**
	 *     
	 * @throws ExtractError
	 * @throws ErrorReadingProject
	 */
	private List<File> extractAll() throws ExtractError, ErrorReadingProject{
		myExtractor.refreshCurrentSkin();
	    
	    ArrayList<File> tempFiles = new ArrayList<File>();
	    ZipFile zip = myExtractor.getCurrentSkin();
		for (ZipEntry entry : myExtractor.getEntries(zip)){
			try{
				File file = myExtractor.extractEntry(zip, entry);
				if (file != null)
					tempFiles.add(file);
			}catch(Exception e){
				e.printStackTrace();
				System.exit(0);
			}
		}
		return Collections.unmodifiableList(tempFiles);
	}
	
	/**
	 *        @param destinationPath
	 *         ,   ,
	 *            
	 *        
	 * @throws WSZWriteException   
	 */
	public void writeAll(String destinationPath) throws WSZWriteException{
		
		List<File> tempFiles = new ArrayList<File>();
		
		/*try{
			(new File(destinationPath)).delete();
		}catch(Exception e){
			throw new WSZWriteException("   ");
		}*/

		try{
			tempFiles = extractAll();
		}catch(Exception e){
			throw new WSZWriteException("    ");
		}
		
		/*try{
			writeTxtFile(TxtFileName.VISCOLOR);
			writeTxtFile(TxtFileName.PLEDIT);
		}catch(IOException e){
			throw new WSZWriteException("    ");
		}*/
		
		ZipOutputStream zos;
		
	    try{
	    	//System.out.println(destinationPath);
			zos = createZipOutputStream(destinationPath);
		}catch(IOException e){
			throw new WSZWriteException("        \n" + e.getMessage());
		}
		
	    try{    
	    	for(File f : tempFiles){
	    		if (!(
	    			f.getName().toUpperCase().equals("THUMBS.DB")
	    			)){
	    			
	    			if(f.isFile()){
	    				addFileToZip(zos, f);
	    			}
	    			
	    		}
	    	}  
	    }catch(Exception e){
	    	throw new WSZWriteException("     ");
	   
	    }finally {
	    
	    	try{
	    		zos.close();
	    	}catch(IOException e){
	    		//throw new WSZWriteException("       ");
	    	}
	    	
	    }
	    
	    try{
	    	myExtractor.refreshCurrentSkin();
	    }catch(ErrorReadingProject e){
	    	throw new WSZWriteException("   ");
	    }
	}
	
	/**
	 *   ,      
	 * @throws WSZWriteException
	 */
	public void writeAll()throws WSZWriteException{
		writeAll(myZipFileName);
	}
	
	
	public SkinWriter(ISkin skin, SkinFilesManager extracter){
		refresh(skin, extracter);
	}
	
	public void refresh(ISkin skin, SkinFilesManager extracter){
		mySkin = skin;
		myExtractor = extracter;
		myZipFileName = myExtractor.getCurrentSkinFileName();
	}
}
